@@ -17,9 +17,11 @@ use std::process::Command;
 ///
 /// The contents of PYTHON_HEADER are added at the beginning of the file.
 ///
-pub(crate) fn call_python3(python_commands: &str, output_dir: &str, filename_py: &str) -> std::io::Result<String> {
+pub(crate) fn call_python3(python_commands: &String, path: &Path) -> Result<String, &'static str> {
     // create directory
-    fs::create_dir_all(output_dir)?;
+    if let Some(p) = path.parent() {
+        fs::create_dir_all(p).map_err(|_| "cannot create directory")?;
+    }
 
     // combine header with commands
     let mut contents = String::new();
@@ -27,13 +29,17 @@ pub(crate) fn call_python3(python_commands: &str, output_dir: &str, filename_py:
     contents.push_str(python_commands);
 
     // write file
-    let filepath = Path::new(output_dir).join(filename_py);
-    let mut file = File::create(&filepath)?;
-    file.write_all(contents.as_bytes())?;
-    file.sync_all()?;
+    let mut file = File::create(path).map_err(|_| "cannot create file")?;
+    file.write_all(contents.as_bytes()).map_err(|_| "cannot write file")?;
+
+    // force sync
+    file.sync_all().map_err(|_| "cannot sync file")?;
 
     // execute file
-    let output = Command::new("python3").arg(&filepath).output()?;
+    let output = Command::new("python3")
+        .arg(path)
+        .output()
+        .map_err(|_| "cannot run python3")?;
 
     // results
     let out = match String::from_utf8(output.stdout) {
@@ -62,13 +68,14 @@ pub(crate) fn call_python3(python_commands: &str, output_dir: &str, filename_py:
 mod tests {
     use super::*;
 
+    const OUT_DIR: &str = "/tmp/plotpy/unit_tests";
+
     #[test]
-    fn call_python3_works() -> Result<(), Box<dyn std::error::Error>> {
-        let commands = "print(\"Python says: Hello World!\")";
-        let out_dir = "/tmp/plotpy";
-        let filename = "call_python3_works.py";
-        let output = call_python3(commands, out_dir, filename)?;
-        let data = fs::read_to_string(Path::new(out_dir).join(filename))?;
+    fn call_python3_works() -> Result<(), &'static str> {
+        let commands = "print(\"Python says: Hello World!\")".to_string();
+        let path = Path::new("call_python3_works.py");
+        let output = call_python3(&commands, &path)?;
+        let data = fs::read_to_string(&path).map_err(|_| "cannot read test file")?;
         let mut correct = String::from(PYTHON_HEADER);
         correct.push_str(&commands);
         assert_eq!(data, correct);
@@ -77,21 +84,33 @@ mod tests {
     }
 
     #[test]
-    fn call_python3_twice_works() -> Result<(), Box<dyn std::error::Error>> {
-        let out_dir = "/tmp/plotpy";
-        let filename = "call_python3_twice_works.py";
+    fn call_python3_create_dir_works() -> Result<(), &'static str> {
+        let commands = "print(\"Python says: Hello World!\")".to_string();
+        let path = Path::new(OUT_DIR).join("call_python3_works.py");
+        let output = call_python3(&commands, &path)?;
+        let data = fs::read_to_string(&path).map_err(|_| "cannot read test file")?;
+        let mut correct = String::from(PYTHON_HEADER);
+        correct.push_str(&commands);
+        assert_eq!(data, correct);
+        assert_eq!(output, "Python says: Hello World!\n");
+        Ok(())
+    }
+
+    #[test]
+    fn call_python3_twice_works() -> Result<(), &'static str> {
+        let path = Path::new(OUT_DIR).join("call_python3_twice_works.py");
         // first
-        let commands_first = "print(\"Python says: Hello World!\")";
-        let output_first = call_python3(commands_first, out_dir, filename)?;
-        let data_first = fs::read_to_string(Path::new(out_dir).join(filename))?;
+        let commands_first = "print(\"Python says: Hello World!\")".to_string();
+        let output_first = call_python3(&commands_first, &path)?;
+        let data_first = fs::read_to_string(&path).map_err(|_| "cannot read test file")?;
         let mut correct_first = String::from(PYTHON_HEADER);
         correct_first.push_str(&commands_first);
         assert_eq!(data_first, correct_first);
         assert_eq!(output_first, "Python says: Hello World!\n");
         // second
-        let commands_second = "print(\"Python says: Hello World! again\")";
-        let output_second = call_python3(commands_second, out_dir, filename)?;
-        let data_second = fs::read_to_string(Path::new(out_dir).join(filename))?;
+        let commands_second = "print(\"Python says: Hello World! again\")".to_string();
+        let output_second = call_python3(&commands_second, &path)?;
+        let data_second = fs::read_to_string(&path).map_err(|_| "cannot read test file")?;
         let mut correct_second = String::from(PYTHON_HEADER);
         correct_second.push_str(&commands_second);
         assert_eq!(data_second, correct_second);
